import os
import json

from django.conf import settings
from django.contrib.auth.decorators import user_passes_test
from django.core.files.storage import default_storage
from django.http import HttpResponse
from django.utils.text import slugify
from django.views.decorators.http import require_POST
from .forms import FileForm


UPLOAD_PATH = getattr(settings, 'AJAXIMAGE_DIR', 'ajaximage/')
AUTH_TEST = getattr(settings, 'AJAXIMAGE_AUTH_TEST', lambda u: u.is_staff)
FILENAME_NORMALIZER = getattr(settings, 'AJAXIMAGE_FILENAME_NORMALIZER', slugify)


@require_POST
@user_passes_test(AUTH_TEST)
def ajaximage(request, upload_to=None, form_class=FileForm):
    form = form_class(request.POST, request.FILES)
    if form.is_valid():
        file_ = form.cleaned_data['file']
        image_types = ['image/png', 'image/jpg', 'image/jpeg', 'image/pjpeg',
                       'image/gif', 'image/svg+xml']
        if file_.content_type not in image_types:
            data = json.dumps({'error': 'Bad image format.'})
            return HttpResponse(data, content_type="application/json", status=403)

        file_name, extension = os.path.splitext(file_.name)
        safe_name = '{0}{1}'.format(FILENAME_NORMALIZER(file_name), extension)
        name = os.path.join(upload_to or UPLOAD_PATH, safe_name)
        path = default_storage.save(name, file_)
        url = default_storage.url(path)
        return HttpResponse(json.dumps({'url': url, 'filename': path}),
                            content_type="application/json")
    return HttpResponse(status=403)



from django.contrib.contenttypes.admin import GenericTabularInline

from adminsortable.admin import SortableGenericTabularInline, NonSortableParentAdmin


class ImageInline(SortableGenericTabularInline, GenericTabularInline):
    model = None
    extra = 0


class AjaxImageUploadMixin(NonSortableParentAdmin):
    change_form_template = 'ajaximage/change_form.html'
    ajax_change_form_template_extends = 'adminsortable/change_form.html'
    image_inline = ImageInline
    upload_to = '/images/'

    # noinspection PyProtectedMember
    def change_view(self, request, object_id, form_url='', extra_context=None):
        if extra_context is None:
            extra_context = {}
        extra_context.update({
            'ajax_change_form_template_extends': self.ajax_change_form_template_extends,
        })
        extra_context.update({
            'upload_to': self.upload_to.strip('/'),
        })
        return super().change_view(request, object_id, form_url='', extra_context=extra_context)

    # noinspection PyProtectedMember
    def add_view(self, request, form_url='', extra_context=None):
        if extra_context is None:
            extra_context = {}
        extra_context.update({
            'ajax_change_form_template_extends': self.ajax_change_form_template_extends,
        })
        extra_context.update({
            'upload_to': self.upload_to.strip('/'),
        })
        return super().add_view(request, form_url='', extra_context=extra_context)

    class Media:
        js = (
            'dropzone/js/dropzone.js',
        )
        css = {
            'all': (
                'dropzone/css/dropzone.css',
                'dropzone/css/basic.css',
            )
        }



from django.db import models
from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.apps import apps
from adminsortable.models import SortableMixin

from .fields import AjaxImageField


# noinspection PyAbstractClass
class AbstractImage(SortableMixin, models.Model):
    order = models.PositiveIntegerField('Порядок', default=0, editable=False, db_index=True)
    file = AjaxImageField('Файл', upload_to='ajaximage/image/file')
    description = models.TextField('Описание', blank=True)

    content_type = models.ForeignKey('contenttypes.ContentType', on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField(editable=False)
    content_object = GenericForeignKey()

    class Meta:
        abstract = True
        ordering = ['order']
        verbose_name = 'Изображение'
        verbose_name_plural = 'Изображения'

    def __str__(self):
        return str(self.order)



import os

from django.forms import widgets
from django.utils.safestring import mark_safe
from django.urls import reverse
from django.core.files.storage import default_storage


class AjaxImageWidget(widgets.TextInput):

    html = """
    <div class="ajaximage">
        <a class="file-link" target="_blank" href="{file_url}">
            <img class="file-img" src="{file_url}">
        </a>
        <a class="file-remove" href="#remove">Remove</a>
        <input class="file-path" type="hidden" value="{file_path}" id="{element_id}" name="{name}" />
        <input type="file" class="file-input" />
        <input class="file-dest" type="hidden" value="{upload_url}">
        <div class="progress progress-striped active">
            <div class="bar"></div>
        </div>
    </div>
    """

    class Media:
        js = (
            'ajaximage/js/ajaximage.js',
        )
        css = {
            'all': (
                'ajaximage/css/bootstrap-progress.min.css',
                'ajaximage/css/styles.css',
            )
        }

    def __init__(self, *args, **kwargs):
        self.upload_to = kwargs.pop('upload_to', '')
        super(AjaxImageWidget, self).__init__(*args, **kwargs)

    # noinspection PyMethodOverriding
    def render(self, name, value, attrs=None):
        final_attrs = self.build_attrs(attrs)
        element_id = final_attrs.get('id')
        kwargs = {'upload_to': self.upload_to}
        upload_url = reverse('ajaximage', kwargs=kwargs)
        file_path = str(value) if value else ''
        file_url = default_storage.url(file_path) if value else ''
        file_name = os.path.basename(file_url)
        output = self.html.format(upload_url=upload_url,
                                  file_url=file_url,
                                  file_name=file_name,
                                  file_path=file_path,
                                  element_id=element_id,
                                  name=name)

        return mark_safe(output)



from django import template

register = template.Library()


@register.filter(name='upload_to')
def upload_to(model):
    fields = model._meta.get_fields()
    field = next((field for field in fields if hasattr(field, 'upload_to')), None)
    if not field:
        return None
    if callable(field.upload_to):
        return field.upload_to(model, '').strip('/')
    return field.upload_to




import posixpath
import datetime

from django.core.files.storage import default_storage
from django.db.models.fields.files import FileDescriptor, FieldFile
from django.db.models import Field
from django.utils.encoding import force_str, force_text

from .widgets import AjaxImageWidget


class AjaxImageField(Field):

    storage = default_storage
    attr_class = FieldFile
    descriptor_class = FileDescriptor

    def __init__(self, *args, **kwargs):
        self.upload_to = kwargs.pop('upload_to', '')
        self.widget = AjaxImageWidget(upload_to=self.upload_to)
        super(AjaxImageField, self).__init__(*args, **kwargs)

    # noinspection PyMethodOverriding
    def contribute_to_class(self, cls, name, virtual_only=False):
        super(AjaxImageField, self).contribute_to_class(cls, name, virtual_only)
        setattr(cls, self.name, self.descriptor_class(self))

    def get_prep_value(self, value):
        if value is None:
            return None
        return str(value)

    def get_internal_type(self):
        return 'TextField'

    def formfield(self, **kwargs):
        defaults = {'widget': self.widget}
        defaults.update(kwargs)
        return super(AjaxImageField, self).formfield(**defaults)

    def generate_filename(self, instance, filename):
        if callable(self.upload_to):
            filename = self.upload_to(instance, filename)
        else:
            dirname = force_text(datetime.datetime.now().strftime(force_str(self.upload_to)))
            filename = posixpath.join(dirname, filename)
        return self.storage.generate_filename(filename)
